@@ -5,9 +5,9 @@ import json
 import sys
 import random
 import traceback
-from queue import Queue as SendQueue
 from functools import update_wrapper
 from threading import Thread
+from threading import RLock
 
 from kombu import Connection
 from kombu import Exchange
@@ -79,7 +79,7 @@ class RabbitMQ(object):
         self.send_connection = None
         self.app = app
         self.message_callback_list = []
-        self.wait_send_queue = None
+        self.wait_send_lock = None
         if app is not None:
             self.init_app(app)
 
@@ -91,7 +91,7 @@ class RabbitMQ(object):
         self.send_connection = self.connection.clone()
         self.send_exchange_name = self.config.get('RABMQ_SEND_EXCHANGE_NAME')
         self.send_exchange_type = self.config.get('RABMQ_SEND_EXCHANGE_TYPE') or ExchangeType.TOPIC
-        self.wait_send_queue = SendQueue(maxsize=1)
+        self.wait_send_lock = RLock()
 
     def run_consumer(self):
         self._run()
@@ -204,34 +204,23 @@ class RabbitMQ(object):
             auto_delete=False,
             durable=True
         )
-        while True:
-            try:
-                self.wait_send_queue.join()
-                self.wait_send_queue.put(body, block=False)
-                break
-            except Exception as E:
-                logger.info('wait lock')
-                pass
-        try:
+        with self.wait_send_lock:
             channel = self.send_connection.default_channel
             exchange.declare(channel=channel)
             self.consumer.producer.publish(
-                body=self.wait_send_queue.get(),
+                body=body,
                 exchange=exchange,
                 routing_key=routing_key,
                 retry=True,
                 headers=headers,
             )
             logger.info(log_flag, 'send data: %s', body)
-        except Exception as E:
-            pass
-        finally:
-            self.wait_send_queue.task_done()
 
     def retry_send(self, body, queue_name, headers=None, log_flag='', **kwargs):
         logger.info(log_flag, 'send data: %s', body)
-        simple_queue = self.consumer.connection.SimpleQueue(queue_name)
-        simple_queue.put(body, headers=headers, retry=True, **kwargs)
+        with self.wait_send_lock:
+            simple_queue = self.send_connection.SimpleQueue(queue_name)
+            simple_queue.put(body, headers=headers, retry=True, **kwargs)
 
     def delay_send(self, body, routing_key, delay=None, exchange_name=None, log_flag=None, **kwargs):
         logger.info(log_flag, 'send data: %s', body)
@@ -240,8 +229,11 @@ class RabbitMQ(object):
             'x-dead-letter-exchange': exchange_name
         }
         queue_name = '%s_%s' % (exchange_name, re.sub('[^0-9a-zA-Z]+', '', routing_key))
-        simple_queue = self.consumer.connection.SimpleQueue(
-            queue_name,
-            queue_args=dead_letter_params
-        )
-        simple_queue.put(body, retry=True, expiration=delay, **kwargs)
+        with self.wait_send_lock:
+            channel = self.send_connection.default_channel
+            simple_queue = self.send_connection.SimpleQueue(
+                queue_name,
+                queue_args=dead_letter_params,
+                channel=channel
+            )
+            simple_queue.put(body, retry=True, expiration=delay, **kwargs)
